

/*********************************************************
 *               Hlavickove soubory pro TUOX             *
 *                                                       *
 * Soubor:    stdio.h                                    *
 * Popis:     Definice proudovych V/V funkci             *
 *                                                       *
 * + makra:   getc()                                     *
 *            putc()                                     *
 *            clearerr()                                 *
 *            ferror()                                   *
 *            feof()                                     *
 *                                                       *
 * Autori:    Rostislav Bosak                            *
 *            Jaroslav Hurdes                            *
 *            Martin Perina                              *
 *            Martin Vojvodik                            *
 *                                                       *
 * Preprac:   Daniel Tvrdy				 *
 *	      Ivo Orlik                         (c) 1997 *
 *                                                       *
 *            Daniel Smolka & Eva Drstakova     (c) 1999 *
 *                                                       *
 *********************************************************/


#define _NULLSTREAM_TEST	// pri kazdem volani stdio provadet if (!stream) quit;


#ifndef _STDIO_H_
#define _STDIO_H_

#include "_null.h"

#define _NFILE_ 20

 #ifndef _SIZE_T
 #define _SIZE_T
  typedef unsigned size_t;
 #endif



/* Size of an arry large enough to hold a temporary file name string */
#define L_ctermid   5       /* CON: plus null byte */
#define P_tmpdir    ""      /* temporary directory */
#define L_tmpnam    15      /* tmpnam buffer size  */
//extern char __tmpnam_buffer[L_tmpnam];


/* Definition of the file position type */
typedef long    fpos_t;


/* Definition of the control structure for streams */
typedef struct  {
        int             level;          /* fill/empty level of buffer */
        unsigned        flags;          /* File status flags          */
        char            fd;             /* File descriptor            */
        unsigned char   hold;           /* Ungetc char if no buffer   */
        int             bsize;          /* Buffer size                */
        unsigned char    *buffer;       /* Data transfer buffer       */
        unsigned char    *curp;         /* Current active pointer     */
        unsigned        istemp;         /* Temporary file indicator   */
        short           token;          /* Used for validity checking */
        char            tmpnam[L_tmpnam]; /* filename if temporary    */
}       FILE;                           /* This is the FILE object    */

/* Bufferisation type to be used as 3rd argument for "setvbuf" function */
#define _IOFBF  0
#define _IOLBF  1
#define _IONBF  2


/*  "flags" bits definitions  - by Borland on PC */
#define _F_RDWR 0x0003                  /* Read/write flag       */
#define _F_READ 0x0001                  /* Read only file        */
#define _F_WRIT 0x0002                  /* Write only file       */
#define _F_BUF  0x0004                  /* Malloc'ed Buffer data */
#define _F_LBUF 0x0008                  /* line-buffered file    */
#define _F_ERR  0x0010                  /* Error indicator       */
#define _F_EOF  0x0020                  /* EOF indicator         */
#define _F_BIN  0x0040                  /* Binary file indicator */
#define _F_IN   0x0080                  /* Data is incoming      */
#define _F_OUT  0x0100                  /* Data is outgoing      */
#define _F_TERM 0x0200                  /* File is a terminal    */


/* End-of-file constant definition */
#define EOF (-1)            /* End of file indicator */

/* Number of files that can be open simultaneously */
#define FOPEN_MAX (_NFILE_)     /* Able to have 20 files */
#define SYS_OPEN  (_NFILE_)

#define FILENAME_MAX 18

/* Default buffer size use by "setbuf" function */
#define BUFSIZ  512         /* Buffer size for stdio */

/* Constants to be used as 3rd argument for "fseek" function */
#define SEEK_CUR    1
#define SEEK_END    2
#define SEEK_SET    0

/* Number of unique file names that shall be generated by "tmpnam" function */
#define TMP_MAX     0xFFFF

/* Standard I/O predefined streams */

#define FD_STDIN  0
#define FD_STDOUT 1
#define FD_STDERR 2

extern FILE
 *stdin,
 *stdout,
 *stderr;

/*=---------------- stdio functions ------------------=*/

int       fclose(FILE  *__stream);
int       fflush(FILE  *__stream);
int       fgetc(FILE  *__stream);
int       fgetpos(FILE  *__stream, fpos_t  *__pos);
char    * fgets(char  *__s, int __n, FILE  *__stream);
FILE    * fopen(const char  *__path, const char  *__mode);
int       fprintf(FILE  *__stream, const char  *__format, ...);
int       fputc(int __c, FILE  *__stream);
int       fputs(const char  *__s, FILE  *__stream);
size_t    fread(void  *__ptr, size_t __size, size_t __n, FILE  *__stream);
FILE    * freopen(const char  *__path, const char  *__mode, FILE  *__stream);
int       fscanf(FILE  *__stream, const char  *__format, ...);
int       fseek(FILE  *__stream, long __offset, int __whence);
int       fsetpos(FILE  *__stream, const fpos_t  *__pos);
long      ftell(FILE  *__stream);
size_t    fwrite(const void  *__ptr, size_t __size, size_t __n, FILE  *__stream);
char    * gets(char  *__s);
void      perror(const char  *__s);
int       printf(const char  *__format, ...);
int       puts(const char  *__s);
int       remove(const char  *__path);
int       rename(const char  *__oldname,const char  *__newname);
void      rewind(FILE  *__stream);
int       scanf(const char  *__format, ...);
int       sprintf(char  *__buffer, const char  *__format, ...);
int       sscanf(const char  *__buffer, const char  *__format, ...);
char    * strerror(int __errnum);
FILE    * tmpfile(void);
char    * tmpnam(char *s);
int       ungetc(int __c, FILE  *__stream);
int       vfprintf(FILE  *__stream, const char  *__format, void  *__arglist);
int       vfscanf(FILE  *__stream, const char  *__format, void  *__arglist);
int       vprintf(const char  *__format, void  *__arglist);
int       vscanf(const char  *__format, void  *__arglist);
int       vsprintf(char * buf, const char * fmt, void  *__arglist);
int       vsscanf(const char  *__buffer, const char  *__format, void  *__arglist);
int       getchar(void);
int       putchar(const int __c);


/*  The following macros provide for common functions */

#define getc(f)   fgetc(f)
#define putc(c,f) fputc(c,f)

#define clearerr(f) ((f)->flags &= !(_F_ERR|_F_EOF))
#define ferror(f)   ((f)->flags & _F_ERR)
#define feof(f)     ((f)->flags & _F_EOF)

#endif

