/*********************************************************
 *                   TUOX - knihovny                     *
 *********************************************************
 * Funkce:    fgetc()                                    *
 *            fputc()                                    *
 *            getchar()                                  *
 *            putchar()                                  *
 *            ungetc()                                   *
 *            				                 *
 * Autori:    Daniel Smolka & Eva Drstakova     (c) 1999 *
 *                                                       *
 * Funkce vraci:                                         *
 *            pocet uspesne prectenych elementu          *
 *********************************************************/

#include "lib\stdio\ffs.h"


int fgetc(FILE  *stream)
{
 unsigned char c;
 int succ;

#ifdef _NULLSTREAM_TEST
 if (!stream) return EOF;
#endif

 if (stream->hold != 0xff)
  {
   stream->flags &= !_F_ERR;	/* OK - read ungotten char */
   c = stream->hold;
   stream->hold = 0xff;
   return (int)c;
  }

 succ = read (stream->fd, &c, 1);
 if (succ == EOF)
  {
   stream->flags |= _F_ERR;		/* error */
   return EOF;
  }

 if (!succ)
  {
   stream->flags |= _F_EOF;		/* eof */
   return EOF;
  }

 stream->flags &= !(_F_ERR|_F_EOF);	/* OK */
 return (int)c;
}


int fputc(int cint, FILE  *stream)
{
 unsigned char c = (unsigned char) cint;
 int succ;

#ifdef _NULLSTREAM_TEST
 if (!stream) return EOF;
#endif

 succ = write(stream->fd, &c, 1);
 if (!succ || succ == EOF)
  {
   stream->flags |= _F_ERR;	/* error */
   return EOF;
  }

 stream->flags &= !_F_ERR;	/* OK */
 return (int)c;
}


int ungetc(int cint, FILE  *stream)
{
 unsigned char c = (unsigned char) cint;

#ifdef _NULLSTREAM_TEST
 if (!stream) return EOF;
#endif
 
 if (c != 0xff)
  {
   stream->hold = c;
   stream->flags &= !(_F_ERR|_F_EOF);	/* OK */
   return (int)c;
  }

 stream->hold = 0xff;
 stream->flags |= _F_ERR;	/* error */
 return EOF;
}


int getchar(void)
{
 return fgetc (stdin);
}


int putchar(const int c)
{
 return fputc (c,stdout);
}

