/*********************************************************
 *                   TUOX - knihovny                     *
 *********************************************************
 * Funkce:    fopen()                                    *
 *            fclose()	                                 *
 *            freopen()                                  *
 *            fflush()                                   * - udelat
 *            				                 *
 *            tmpfile()                                  *
 *            tmpnam()                                   * - dodelat rnd()
 *            				                 *
 *            remove()                                   *
 *            rename()                                   * - dodelat
 *            				                 *
 * Detaily:   stdio.html                                 *
 *            				                 *
 * Autor:     Martin Vojvodik	                         *
 *	      Bosak Rostislav                            *
 *            Hurdes Jaroslav                            *
 *            Perina Martin                              *
 * 							 *
 * Preprac:   Daniel Tvrdy 			         *
 *            Ivo Orlik	                        (c) 1997 *
 *                                                       *
 * Prepsali:  Daniel Smolka & Eva Drstakova     (c) 1999 *
 *                                                       *
 *********************************************************/


#include "lib\stdio\ffs.h"

char __tmpnam_buffer[L_tmpnam];

FILE* fopen(const char *path, const char *mode)
{
  register FILE *stream;
  int delka, citac,command,append,trunc;
  unsigned int openflags;
  char *c;

  /* allokace struct FILE */
  stream = (FILE*) malloc (sizeof(FILE));
  if (!stream) return NULL;

  /* analyza modu */
  c = (char*)mode;
  delka = strlen(c);

  stream->hold  = 0xff;		/* empty ungetc() buffer */
  stream->flags = 0;
  openflags = 0;
  command = 0;
  append = 0;
  trunc = 0;


/*  tak takhle to chodi podle ANSI standardu:

  for (citac = 0; citac < delka; citac++)
   {
    switch (*c) {
     case 'r' :  command |= 1; break;
     case 'w' :  command |= 2; break;
     case 'a' :  command |= 4; break;
     case '+' :  command |= 8; break;
     case 'b' :  stream->flags |= _F_BIN;
		 openflags |= O_BINARY; break;		// "b"
     }
    c++;
   }

  switch (command) {
	case 1:  openflags |= O_RDONLY;          break;	// "r"
	case 2:  openflags |= O_WRONLY|O_TRUNC;  break;	// "w"
	case 4:  openflags |= O_APPEND|O_WRONLY; break;	// "a"
	case 3:                                   	// "rw"
	case 9:  openflags |= O_RDWR;            break;	// "r+"
	case 10: openflags |= O_RDWR|O_TRUNC;    break;	// "w+"
	case 12: openflags |= O_RDWR|O_APPEND;   break;	// "a+"
	default:					// "??"
	    free (stream);
	    return NULL;	// zadany nesmyslny mod
  }

  konec ANSI sekce  */


/* a takhle to chodi po nasemu v TUOXu:  */

  for (citac = 0; citac < delka; citac++)
   {
    switch (*c) {
     case 'r' :  command |= 1; break;
     case 'w' :  command |= 2; break;
     case 'a' :  command |= 4; append=1; break;
     case '+' :  command |= 8; break;
     case 'b' :  break;                    // not supported ???  :-(
     }
    c++;
   }
  openflags |= I_REGULAR;

  switch (command) {
	case 1:  openflags |= S_IREAD;                   break;	// "r"
	case 2:  openflags |= S_IWRITE; trunc=1;         break;	// "w"
	case 4:  openflags |= S_IWRITE;                  break;	// "a"
	case 3:                                   	        // "rw"
	case 9:  openflags |= S_IREAD|S_IWRITE;          break;	// "r+"
	case 10: openflags |= S_IREAD|S_IWRITE; trunc=1; break;	// "w+"
	case 12: openflags |= S_IREAD|S_IWRITE;          break;	// "a+"
	default:						// "??"
	    free (stream);
	    return NULL;	// zadany nesmyslny mod
  }

  /* konec TUOX portace */


  /* alokace bufferu */
  stream->buffer = (unsigned char *) malloc(BUFSIZ);
  if (!stream->buffer)
   {
    free (stream);
    return NULL;       /* malo pameti pro buffer */
   }
  stream->bsize = BUFSIZ;      /* BUFSIZ definovan v stdio.h */
  stream->flags |= _F_BUF;
  stream->istemp = 0;
  stream->fd = -1;

  if (!trunc) stream->fd = open(path, openflags);
  if (stream->fd == -1)
   {
    stream->fd = creat(path, openflags);
   }

  if (stream->fd == -1) {
      /* soubor se nepodarilo otevrit */
   free (stream->buffer);
   free (stream);
   return NULL;
  }

  if (append) fseek (stream, 0L, SEEK_END);

  stream->token = 1;		/* indikace, ze soubor je otevreny */
  return stream;		/* uspesne otevreny soubor */
}



int fclose(FILE * stream)
 {
  int fflag;          /* indikator uzavreni souboru */

#ifdef _NULLSTREAM_TEST
  if (!stream) return EOF;	/* no stream */
#endif

  /* *** */
  /* Volani FS, pro uzavreni souboru */
  fflag = close(stream->fd);

  if (fflag == 0)    /* predpokladame, ze soubor se podarilo zavrit */
   {

   if (stream->istemp) remove ((const char *)&stream->tmpnam);

    stream->token = 0;         /* indikace uzavreneho souboru */

    /* dealokace bufferu */
    if (stream->buffer) free(stream->buffer);
    free (stream);

    return 0;                      /* soubor se podarilo zavrit */
   }
  else return EOF;		/* nejaka chyba pri zavirani */
}



FILE* freopen(const char *path, const char *mode, FILE *stream)
{
#ifdef _NULLSTREAM_TEST
 if (!stream) return NULL;		/* sorry - no handle */
#endif

 if (fclose (stream) == EOF) return NULL;  /* nelze zavrit */
 return fopen (path, mode);
}



int fflush(FILE *stream)
{
 sync();	// osizene - flush uplne vseho
 return 0;
}



char __tmpnam_buffer[L_tmpnam];		/* buffer for tmpnam() */

char *tmpnam(char *s)
{
 int i;
 long cislo;
 char t;

 if (!s) s = (char*)&__tmpnam_buffer;

/* problem zjisteni nahodneho cisla - asi podle casu */

 cislo = 0L;
 time (&cislo);				/* nejake jedno jake cislo */

 cislo += (long)s + 0x1234abcf;		/* nejake nahodne cislo */

 for (i=0; i<8; i++) {
  t = (char)(cislo&0x1f);
  cislo >>= 4;
  if (t < 10) t+= '0';
  else t += 'A'-10;
  *(s+7-i) = t;
 }
 strcpy(s+8,".tmp");			/* filename suffix */
 return s;
}


FILE *tmpfile(void)
{
 char bufS[L_tmpnam], *buf = (char*)&bufS;

 register FILE *stream;
 unsigned char t;
 int i;
 long cislo;

 /* allokace struct FILE */
 stream = (FILE*) malloc (sizeof(FILE));
 if (!stream) return NULL;

 cislo = (long)stream;		/* nejake nahodne cislo */
 for (i=0; i<8; i++) {
  t = (char)(cislo&0x1f);
  cislo >>= 4;
  if (t < 10) t+= '0';
  else t += 'A'-10;
  *(buf+7-i) = t;
 }
 strcpy(buf+8,".tmp");			/* filename suffix */

 stream->bsize = BUFSIZ;		/* BUFSIZ definovan v stdio.h */
 stream->flags |= _F_BUF;		/* je bufferovan a temporary */
 stream->hold  = 0xff;			/* empty ungetc() buffer */
 stream->istemp = 1;			/* yes, this file is temporary */
 strcpy((char*)&stream->tmpnam, buf);	/* store the temp. filename */

 stream->fd = creat(buf, I_REGULAR|S_IREAD|S_IWRITE);	// "wb+" mode

  if (stream->fd == -1) {		/* soubor se nepodarilo otevrit */
   free (stream->buffer);
   free (stream);
   return NULL;
  }

 stream->token = 1;		/* indikace, ze soubor je otevreny */
 return stream;			/* uspesne otevreny soubor */
}



int remove(const char *link)
{
 return unlink (link);
}


int rename(const char  *oldname,const char  *newname)
{
 int errcode = link (oldname, newname);
 if (errcode) return errcode;

 return unlink (oldname);
}


FILE
__stdinS  = {0, _F_READ|_F_LBUF|_F_TERM, FD_STDIN,  0xff, 0, NULL, NULL, 0, 1, ""},
__stdoutS = {0, _F_WRIT|_F_LBUF|_F_TERM, FD_STDOUT, 0xff, 0, NULL, NULL, 0, 1, ""},
__stderrS = {0, _F_WRIT|_F_LBUF|_F_TERM, FD_STDERR, 0xff, 0, NULL, NULL, 0, 1, ""},

*stdin  = &__stdinS,
*stdout = &__stdoutS,
*stderr = &__stderrS;
