/*********************************************************
 *                   TUOX - knihovny                     *
 *********************************************************
 *            				                 *
 * Soubor:    fseek.c - seekovanci funkce                *
 *            				                 *
 * Funkce:    fseek()                                    *
 *            ftell()                                    *
 *            fgetpos()                                  *
 *            fsetpos()                                  *
 *            rewind()                                   *
 *            				                 *
 * Detaily:   stdio.htm                                  *
 *            				                 *
 * Autor:     Martin Vojvodik	                         *
 *	      Bosak Rostislav                            *
 *            Hurdes Jaroslav                            *
 *            Perina Martin                              *
 * 							 *
 * Preprac:   Daniel Tvrdy 			         *
 *            Ivo Orlik	                        (c) 1997 *
 *                                                       *
 * Prepsali:  Daniel Smolka & Eva Drstakova     (c) 1999 *
 *                                                       *
 *********************************************************/

#include "lib\stdio\ffs.h"


int fseek(FILE * stream, long offset, int whence)
{
  int succ = -1;

#ifdef _NULLSTREAM_TEST
  if (!stream) return -1;	 /* soubor neni otevren */
#endif

  if (lseek (stream->fd, offset, whence) != -1) {
   succ = 0;				/* OK */
   stream->flags &= !(_F_EOF|_F_ERR);	/* clr EOF|ERR flags */
  }
  else stream->flags |= _F_ERR;		/* set ERR flag */

  return succ;

} /* fseek(FILE*,long,int) */



long ftell(FILE  *stream)
{
  long pos;

#ifdef _NULLSTREAM_TEST
  if (!stream) return -1; 	/* no stream */
#endif

  pos = fseek(stream, 0L, SEEK_CUR);

  if (pos == -1L) stream->flags |= _F_ERR;	/* set ERR flag */
  else stream->flags &= !_F_ERR;		/* clr ERR flag */

  return pos;
}



int fgetpos(FILE  *stream, fpos_t  *pos)
{
#ifdef _NULLSTREAM_TEST
  if (!stream) return -1; 	/* no stream */
#endif

  *pos = lseek(stream->fd, 0L, SEEK_CUR);

  if (*pos == -1L) {
   stream->flags |= _F_ERR;	/* set ERR flag */
   return EOF;
  }

  stream->flags &= !_F_ERR;	/* clr ERR flag */
  return 0;
}



int fsetpos(FILE  *stream, const fpos_t  *pos)
{
  long status;

#ifdef _NULLSTREAM_TEST
  if (!stream) return -1; 	/* no stream */
#endif

  status = lseek(stream->fd, *pos, SEEK_SET);

  if (status == -1L) {
   stream->flags |= _F_ERR;	/* set ERR flag */
   return EOF;
  }

  stream->flags &= !(_F_EOF|_F_ERR);	/* clr EOF|ERR flags */
  return 0;
}


void rewind(FILE *stream)
{
#ifdef _NULLSTREAM_TEST
 if (!stream) return -1; 	/* no stream */
#endif

 lseek (stream->fd, 0L, SEEK_SET);

 stream->flags &= !(_F_EOF|_F_ERR);	/* clr EOF|ERR flags */
}
