/*********************************************************
 * funkce:    fscanf()                                   *
 *	      scanf()               	         	 *
 *	      sscanf()               	         	 *
 *	      vfscanf()               	         	 *
 *	      vscanf()               	         	 *
 *	      vsscanf()               	         	 *
 *                                                       *
 * Aurori:  Eva Drstakova & Daniel Smolka, 1999 	 *
 *********************************************************/


#include "lib\stdio\ffs.h"



char bufS[BUFSIZ], *scanbuf = (char*)&bufS;		/* 512 B buffer */


int fscanf(FILE * stream, const char * fmt, ...)
{

   va_list args;
   int i;

   fgets((char *)scanbuf, BUFSIZ, stream);

   va_start (args, fmt);
   i=vsscanf(scanbuf, fmt, args);
   va_end (args);

   return i;
}


int scanf(const char * fmt, ...)
{

   va_list args;
   int i;

   gets((char *)scanbuf);

   va_start (args, fmt);
   i=vsscanf(scanbuf, fmt, args);
   va_end (args);

   return i;
}


int sscanf(const char *buf, const char * fmt, ...)
{
   va_list args;
   int i;

   va_start (args, fmt);
   i=vsscanf(buf, fmt, args);
   va_end (args);

   return i;
}


int vfscanf(FILE *stream, const char *fmt, va_list args)
{
   fgets((char *)scanbuf, BUFSIZ, stream);
   return vsscanf((char *)scanbuf, fmt, args);
}


int vscanf(const char * fmt, va_list args)
{

   int i;

   gets((char *)scanbuf);

   va_start(args, fmt);
   i=vsscanf(scanbuf, fmt, args);
   va_end(args);

   return i;
}


/*=-----------------------------------------------------------=*/
/*							       */
/*    zacatek vsscanf()					       */
/*							       */

/*
size_t strnlen(const char *  s, size_t count)
{
   const char * sc;
   for (sc = s; count-- && *sc != '\0'; ++sc)
   return sc - s;
}
*/


/* we use this so that we can do without the ctype library */
#define is_digit(c)  ((c) >= '0' && (c) <= '9')



static int skip_atoi(const char * *s)
{
   int i=0;

   while (is_digit(**s))
      i = i*10 + *((*s)++) - '0';
   return i;
}



#define ZEROPAD  1 	/* pad with zero */
#define SIGN  2	 	/* unsigned/signed long */
#define PLUS  4 	/* show plus */
#define SPACE  8	/* space if plus */
#define LEFT  16 	/* left justified */
#define SPECIAL  32 	/* 0x */
#define LARGE  64 	/* use 'ABCDEF' instead of 'abcdef' */

int vsscanf(const char * buf, const char * fmt, va_list  args)
{
   unsigned long num;
   int i, p=0,base,cnt=0;
   char *str;
   char *s;

 char *cp;
 int *ip;
 unsigned int *uip;
 long lv, *lp;
 unsigned long ulv, *ulp;

   int flags;      		/* flags to number() */

   int field_width;   		/* width of output field */
   int precision;      		/* min. # of digits for integers; max */
				/*   number of chars for from string */
   int qualifier;      		/* 'h', 'l', or 'L' for integer fields */

   for (str = s ; *fmt ; ++fmt) {
   *s = '\0';
      if (*fmt != '%') {
	 continue;
      }
/******************************************/

     for(str = s ; *buf; ++buf){

	  if(*buf != ' '){
		*str++ = *buf;
		p = 1;

	  }
	  else if (p == 1){
		p = 0;
		break;
	       }
	       else continue;
     }
     *str++ = '\0';




/*******************************************/

      /* process flags */
      flags = 0;
      repeat:
	 ++fmt;      			/* this also skips first '%' */
	 switch (*fmt) {
	    case '-': flags |= LEFT; goto repeat;
	    case '+': flags |= PLUS; goto repeat;
	    case ' ': flags |= SPACE; goto repeat;
	    case '#': flags |= SPECIAL; goto repeat;
	    case '0': flags |= ZEROPAD; goto repeat;
	    }

      /* get field width */
      field_width = -1;
      if (is_digit(*fmt))
	 field_width = skip_atoi(&fmt);
      else if (*fmt == '*') {
	 ++fmt;
	 /* it's the next argument */
	 field_width = va_arg(args, int);
         cnt++;
	 if (field_width < 0) {
	    field_width = -field_width;
	    flags |= LEFT;
	 }
      }

      /* get the precision */
      precision = -1;
      if (*fmt == '.') {
	 ++fmt;
	 if (is_digit(*fmt))
	    precision = skip_atoi(&fmt);
	 else if (*fmt == '*') {
	    ++fmt;
	    /* it's the next argument */
	    precision = va_arg(args, int);
	    cnt++;
	 }
	 if (precision < 0)
	    precision = 0;
      }

      /* get the conversion qualifier */
      qualifier = -1;
      if (*fmt == 'h' || *fmt == 'l' || *fmt == 'L') {
	 qualifier = *fmt;
	 ++fmt;
      }

      /* default base */
      base = 10;

      switch (*fmt) {
      case 'c':
	cp = va_arg(args, char*);
	*cp = *s;
	cnt++;
	continue;

      case 's':
	strcpy( va_arg(args,char*) ,s);
	cnt++;
	continue;


      /* integer number formats - set up the flags and "break" */
      case 'o':
	 base = 8;
	 break;

      case 'X':
      case 'x':
	 base = 16;
	 break;

      case 'd':
      case 'i':
	 flags |= SIGN;
      case 'u':
	 break;

      default:
	 if (*fmt != '%')
	    *str++ = '%';
	 if (*fmt)
	    *str++ = *fmt;
	 else
	    --fmt;
	 continue;
      }


      if (qualifier == 'l')
	  {
	     if (flags & SIGN) {
	      lv = strtol(s,NULL,base);
	      lp = va_arg(args, long *);
	      cnt++;
	      *lp = lv;
	     }
	     else {
	      ulv = strtol(s,NULL,base);
	      ulp = va_arg(args, unsigned long *);
	      cnt++;
	      *ulp = ulv;
	     }
	   }

      else if (qualifier == 'h' || qualifier == -1)
	{
	     if (flags & SIGN) {
	      lv = strtol(s,NULL,base);
	      ip = va_arg(args, int *);
	      *ip = (int)lv;
	      cnt++;
	     }
	     else {
	      ulv = strtol(s,NULL,base);
	      uip = va_arg(args, unsigned int *);
	      *uip = ulv;
	      cnt++;
	      }
	 }

   }/* for */
   *str = '\0';
   *s = '\0';

  return cnt;
}
