/*********************************************************
 * Funkce:    atol.c                                     *
 * Popis:     Prevede retezec s na cislo typu long       *
 *********************************************************
 * Funkce:    atoi.c                                     *
 * Popis:     Prevede retezec s na cislo typu int	 *
 *********************************************************
 *            				                 *
 * Autor:     Martin Vojvodik, 1997                      *
 *							 *
 * Preprac:   Daniel Tvrdy, 1997			 *
 *********************************************************/


#include "LIB\stdlib.h"
#include "LIB\ctype.h"



/*--------------------------------------------------------------------------*
Name            atol - converts a string to an integer

Usage           long atol(const char *nptr);

Prototype in    stdlib.h

Description     Convert a string to a long integer.  The syntax of
                the string must be:

                        long     ::= [isspace]* [sign] digit [digit]*

                Only decimal integers are acceptable.

                Error handling is poor.  The function will protect
                itself (crash-proof) but there is no defined method
                to return an error indication to the caller.  The
                result is undefined if the input string is invalid.

Return value    converted long value of the input string.  If the string
		cannot be converted to a long, the return value is 0.

*---------------------------------------------------------------------------*/

long atol(const char *strP)
{
  int change_sign;
  long result;

  result = 0;
  change_sign = 0;
  while (isspace(*strP++));    /* nejprve odstranime CTYPE mezery zleva */
  if (!*strP)
    return 0;                  /* zpracujeme znamenko */
  if (*strP == '-')
  {
    change_sign = 1;
    strP++;
  }
  if (*strP == '+')
    strP++;
  while (isdigit(*strP))        /* dokud je to cislice */
  {                             /* budeme konvertit */
    result *= 10;               /* decimal radix */
    result += *strP;
    strP++;
  }
  if (change_sign)              /* aplikace znamenka */
    result = -result;
  return result;
}


/*--------------------------------------------------------------------------*

Name            atoi - converts a string to an integer

Usage           int atoi(char *nptr);

Prototype in    stdlib.h

Description     Convert ASCII string to word integer.

                The only difference between this and the atol
                function is whether the result is truncated.

Return value    converted long value of the input string.  If the string
                cannot be converted to an int, the return value is 0.

*---------------------------------------------------------------------------*/

int atoi (const char *strP)
{
        return (int) atol (strP);
}

