/*********************************************************
 * Funkce:    malloc.c                                   *
 * Popis:     Alokace pameti		                 *
 *********************************************************
 * Funkce:    free.c                                     *
 * Popis:     Dealokace pameti		                 *
 *********************************************************
 * Funkce:    calloc.c                                   *
 * Popis:     Alokace pole s nulovanim prvku		 *
 *********************************************************
 * Funkce:    realloc.c                                  *
 * Popis:     Realokace pameti s kopirovanim obsahu      *
 *********************************************************
 *            				                 *
 * Autor:     Martin Vojvodik, 11.6.1997                 *
 *							 *
 * Preprac:   Daniel Tvrdy, 1997			 *
 *********************************************************/

#include "LIB\include\stdlib.h"
#include "LIB\include\process.h"
#include "COMMON\type.h"         /* kvuli typu Word */

/* maximalni argument brk() */
#define __MM_BRKINCR_TOPV       31996        
/* minim. velikost bloku, vznikleho jako prebytek pri alokaci */
#define __MIN_FBLOCK_SIZE       10           

Word __dynastart = 1;    /* zacatek dynamicke pameti */
Word __dynaend = 0;      /* konec doposud obsazene dynamicke pameti */



/*@@@ dopsano dodatecne - nedela nic, jenom vraci nulu */

void* brk(int i) {
 return 0;
}


/*-------------------------------------------------------------------------*

Name            calloc - array allocate

Usage           void *calloc(size_t __ecnt, size_t __esize);

Prototype in    stdlib.h

Description     The calloc function allocates space for an array of __ecnt
		objects, each of whose size is __esize .  The space is
		initialized to all bits zero.


Return value    returns pointer to allocated memory if successful, NULL
		if there is not enough free memory to allocate or if some
		other error occurs.

/*-------------------------------------------------------------------------*/

void * calloc(size_t __ecnt, size_t __esize)
{
  void *retv;
  int i;

  retv = malloc(__ecnt*__esize);
  if (retv != NULL)
    for(i = 0; i<__ecnt*__esize; i++)
      ((char *) retv)[i] = 0;
  return retv;
}



/*-------------------------------------------------------------------------*

Name            malloc - memory allocate

Usage           void *malloc(size_t __size);

Prototype in    stdlib.h

Description     allocate specified amount of dynamic memory.
		Amount in bytes is specified by __size parametr.

Return value    returns pointer to allocated memory if successful, NULL
		if there is not enough free memory to allocate or if some
		other error occurs.

/*-------------------------------------------------------------------------*/

void *malloc(size_t __size)
{
  size_t size, sizep;
  int allocated;
  Word *nowPtr;
  void *retp;


  /* zjistime zda alokovany blok ma velikost do maxima pro brk */
  if (__size > __MM_BRKINCR_TOPV)
    return NULL;

  /* pokud je podminka splnena, tak doposud nebyla provedena alokace */
  if (__dynastart > __dynaend)
  {
    __dynastart = (Word) brk(0);
    __dynaend = __dynastart;
  }

  allocated = 0;
  sizep = 0;

  /* projdeme vsechny bloky, jestli neni nejaky vhodny volny */
  nowPtr = (Word *) __dynastart;
  while ((Word) nowPtr < __dynaend)
  {
    if ((*nowPtr & 0xFFFE) != (((Word)(nowPtr+2)) & 0xFFFE))
    { /* v retezu dynamicke pameti je chyba */
      return NULL;
    }
    size = *(nowPtr + 1);
    if (!(*nowPtr & 0x0001))
    { /* blok je volny */
      if (size >= __size)
      { /* a ma dostacujici velikost */
        if (size < sizep || sizep == 0)
        { /* a doposud jsme nenalezli lepsi blok */
          retp = nowPtr;
          sizep = size;
          allocated = 1;
        }
      }
    }
    nowPtr = (Word *)((Byte *)nowPtr + size + 4);
  }
  if (allocated)
  { /* nalezli jsme pouzitelny volny blok */
    nowPtr = (Word *)retp;
    *nowPtr |= 0x0001;                /* nahodime used bit */
    if ((sizep - __size) > __MIN_FBLOCK_SIZE)
    { /* ma smysl vytvorit novy volny blok z prebyvajiciho prostoru */
      /* zpracujeme puvodni blok */
      *(nowPtr+1) = (Word) __size;    /* zmenime velikost puvodniho bloku */
      retp = (void *)(nowPtr+2);      /* nav.h. je ukazatel za zahlavi */
      nowPtr = (Word *)((Byte *)retp + __size); /* posuneme se za blok*/
      /* v prebyvajici casti vytvorime novy blok */
      *nowPtr = (Word)(nowPtr+2);     /* do zahlavi dame adresu */
      *nowPtr &= 0xFFFE;              /* shodime used bit */
      *(nowPtr+1) = sizep - __size - 4; /* do zahlavi dame velikost */
    }
    else
    { /* nema to smysl */
      *(nowPtr+1) = (Word) sizep;
      retp = (void *)(nowPtr+2);
    }
  }
  else
  { /* vhodny vilny blok nebyl nalezen*/
    if (brk(__size+4) < 0)              /* neni dost pameti */
      return NULL;
    __dynaend += __size + 4;       /* posun ukazatele na konec dyn pameti */
    *nowPtr = (Word)(nowPtr+2);    /* do zahlavi dame adresu */
    *nowPtr |= 0x0001;             /* nastavime used bit na 1 */
    *(nowPtr+1) = __size;          /* do zahlavi dame velikost */
    retp = (void *)(nowPtr+2);     /* nav.h. je ukazatel za zahlavi */
  }

  return retp;
}



/*-------------------------------------------------------------------------*

Name            realloc - memory reallocate

Usage           void *realloc(void * __ptr, size_t __size);

Prototype in    stdlib.h

Description     The realloc function changes the size of the object pointed
                to by ptr to the size specified by size. The contents of
                the object shall be unchanged up to the lesser of the new
                and old sizes.  If the new size is larger, the value of the
                newly allocated portion of the object is indeterminate.
                If ptr is a null pointer, the realloc function behaves like
                the malloc function for the specified size. If the space
                cannot be allocated, the object pointed to by ptr is
                unchanged.  If size is zero and ptr is not a null pointer,
                the object it points to is freed.

Return value    The realloc function returns either a null pointer or a
                pointer to the possibly moved allocated space.


/*-------------------------------------------------------------------------*/

void *realloc(void * __ptr, size_t __size)
{
  size_t size, sizep;
  int i;
  Word *lastPtr, *nowPtr;
  void *tmpp;

  /* jestli je pozadovana velikost vetsi nez parametr pro brk, pak chyba */
  if (__size > __MM_BRKINCR_TOPV)
    return NULL;

  /* jestli byl puvodni ukazatel NULL, provede se klasicky malloc */
  if (__ptr == NULL)
    return malloc(__size);

  /* pokud je pozadovana velikost 0, uvolni se alokovana pamet */
  if (size == 0) {
    free(__ptr);
    return NULL;
  }

  /* __ptr neni NULL, ale dosud nebyla provadena zadna alokace => chyba */
  if (__dynastart >= __dynaend)
  {
    return NULL;
  }

  nowPtr = (Word *) __ptr;
  nowPtr -= 2;                            /* nowPtr ukazuje na zahlavi */

  /* zadany ukazatel nepatri do retezu nebo je ten porusen => chyba */
  if ((*nowPtr & 0xFFFE) != (((Word)__ptr) & 0xFFFE))
    return NULL;

  /* zadany ukazatel ukazatel ukazuje na volny blok => chyba */
  if (!(*nowPtr & 0x0001))
    return NULL;

  size = *(nowPtr+1);

  /* stavajici velikost je rovna pozadovane, neni treba provadet realokaci */
  if (__size <= size && __size > (size - __MIN_FBLOCK_SIZE))
    return __ptr;

  /* pozadovana velikost je mensi nez stavajici, zmensime blok */
  if (__size < size)
  {
    *(nowPtr+1) = __size;            /* do zahlavi dame novou velikost */
    nowPtr = (Word *)((Byte *)__ptr + __size); /* posuneme se za blok */
    /* vytvorime novy blok z uvolneneho prostoru */
    *nowPtr = (Word) nowPtr;         /* do zahlavi dame ukazatel */
    *nowPtr |= 0x0001;               /* nahodime used bit */
    *(nowPtr+1) = (Word)(size-__size-4);  /* do zahlavi dame velikost */
    free(nowPtr);                    /* a klasicky uvolnime tento blok */
    return __ptr;                    /* vratime puvodni ukazatel */
  }

  /* pozadovana velikost je vetsi nez stavajici */
  /* nejprve se pokusime rozsirit blok */
  tmpp = (void *)((Byte *)__ptr + size);  /* tmpp ukazuje na nasled. blok */
  if ((Word)tmpp > __dynaend)        /* jestlize ukazuje za konec d.p. */
    return NULL;                     /* alokacni retez je porusen */
  /* pokud je to posledni blok muze prialokovat dalsi pamet */  
  if ((Word)tmpp == __dynaend)       /* __ptr ukazuje na posledni blok */
  {                                  /* v alokacnim retezu */
    sizep = __size - size;              
    if (sizep < 5)                   /* kazdy blok ma alespon (zahlavi+1)B */
      sizep = 5;                     /* tedy 5 */
    tmpp = malloc(sizep - 4);        /* prialokujeme rozdil */
    if (!tmpp)                       /* neslo prialokovat dalsi pamet */
      return NULL;
    *(nowPtr+1) = (Word)(size+sizep);  /* zmenime velikost bloku v zahlavi */
    return __ptr;                    /* vratime puvodni ukzatel */
  }  
  if (*((Word *)tmpp) & 0xFFFE != (Word)((Byte *)tmpp + 4) & 0xFFFE)
    return NULL;                     /* tmpp adresa neni platna */
  /* pokud nasleduje dostatecne velky volny blok, muzeme je spojit */
  if (!(*((Word *)tmpp) & 0x0001) && *((Word *)tmpp+1)+size+4 >= __size)
  {                                  /* nasledujici blok je volny a vhodny */
    sizep = *((Word *)tmpp+1)+size+4;  /* velikost real. b. je souctem */
    /* pokud ma smysl vytvorit novy volny blok, tak jej vytvorime */
    if ((sizep - __size) > __MIN_FBLOCK_SIZE) 
    {                                     
      tmpp = (void *)((Byte *)tmpp+(__size-size)); /* *tmpp bude novy blok */
      *((Word *)tmpp) = (Word) tmpp + 4;  /* do zahlavi dame adresu */
      *((Word *)tmpp) &= 0xFFFE;          /* shodime used bit */
      *((Word *)tmpp+1) = sizep - __size - 4; /* do zahlavi dame velikost */
      sizep = __size;                     /* velikost real. b. je __size */
    }
    *(nowPtr+1) = (Word) sizep;      /* do zahlavi dame realok. velikost */
    return __ptr;                    /* a vratime ukzatel */
  }
  /* pokud predchazi dostatecne velky blok, muzeme je spojit */
  nowPtr = (Word *) __dynastart;
  while ((Word)nowPtr < __dynaend)
  {
    lastPtr = nowPtr;
    nowPtr = (Word *)((Byte *) lastPtr + *(lastPtr+1) + 4);
    if ((Word)nowPtr >= __dynaend)
      return NULL;
    if (nowPtr+2 == (Word *)__ptr && *lastPtr & 0x0001)
    {
      if (*(lastPtr+1)+4 >= __size - size)
      {
	sizep = *(lastPtr+1)+size+4;
	for(i = 0 ; i < size ; i++ )
          *((Byte *)lastPtr+4+i) = *((Byte *)nowPtr+4+i);
        if (sizep - __size >= __MIN_FBLOCK_SIZE)
        {
	  *(lastPtr+1) = __size;
          nowPtr = (Word *)((Byte *)lastPtr + 4 + __size);
          *nowPtr = (Word)(nowPtr+2);
          *nowPtr &= 0xFFFE;
          *(nowPtr+1) = (Word)(sizep-__size-4);
        }
        *(lastPtr+1) = (Word) sizep; 
        *lastPtr |= 0x0001;
        return (void *)(lastPtr+2);
      } 
    }
  }
  
  /* pokud nelze nic spojit je treba kompletne prealokovat */    
  tmpp = malloc(__size);
  if (tmpp == NULL)
    return NULL;
  for( i = 0 ; i < size ; i++ )
    *((Byte*)tmpp+i) = *((Byte *)__ptr+i);
  return tmpp;     
}



/*-------------------------------------------------------------------------*

Name            free

Usage           void free(void *__block);

Prototype in    stdlib.h

Description     deallocates block of dynamic memory which belongs to
                pointer specified by __block

Return value    none

/*-------------------------------------------------------------------------*/

void free(void *__block)
{
  Word *nowPtr, *lastPtr;                                                   
  size_t size, sizep;                                                      
  int lastfree = 0;

  /* neni nic naalokovano => chyba */
  if (__dynastart >= __dynaend)
    return;                               

  /* ukazatel je NULL => chyba */
  if (__block == NULL)
   return;                                

  nowPtr = (Word *) __block;
  nowPtr -= 2;                       /* nowPtr ukazuje na zahlavi bloku */

  /* zahlavi neobsahuje adresu => blok neni platny => chyba */
  if ((*nowPtr & 0xFFFE) != ((Word) __block & 0xFFFE))
    return;

  __block = (Word *)__block - 2;     /* __block taky */

  /* blok je volny => chyba */
  if (!(*nowPtr & 0x0001))
    return;                          

  *nowPtr &= 0xFFFE;                 /* shodime used bit */

  /* projdeme retez dynamicke pameti */
  nowPtr = (Word *) __dynastart;
  while ((Word)nowPtr < __dynaend)
  {
    lastfree = 0;                         /* posledni je obsazen */
    if (!(*nowPtr & 0x0001))              /* pokud je aktualni blok volny */
    {
      lastPtr = nowPtr;                   /* lastptr ukazuje na tento blok */
					  /* nowptr ukazuje na dalsi blok */
      nowPtr = (Word *)((Byte *)nowPtr+*(lastPtr+1)+4);
      if ((void *)nowPtr == __block)      /* dalsi blok je zadany blok */
      {
	__block = (void *) lastPtr;       /* tak je spojime */
	*(lastPtr+1) += *(nowPtr+1)+4;    /* spojeni: soucet velikosti */
					  /* nowptr ukazuje na dalsi blok */
	lastfree = 1;                     /* posledni byl volny */
      }
      else
      if ((void *)lastPtr == __block )
      {                                   /* aktualni blok je zadany blok */
	if ((Word) nowPtr < __dynaend && !(*(nowPtr) & 0x0001))
	{                                 /* dalsi blok je pritom volny */
	  *(lastPtr+1) += *(nowPtr+1)+4;  /* tak je spojime */
	}
	lastfree = 1;                     /* posledni byl volny */
      }
    }
    nowPtr = (Word *)((Byte *)nowPtr + *(nowPtr+1)+4); /* dalsi blok */
  }

  /* pokud je posledni blok alokacniho retezu volny, posuneme BRK zpet */
  if (lastfree)
  {
    lastPtr = (Word *)__block;
    size = *(lastPtr+1)+4;
    if ((__dynaend - size) < __dynastart)  /* nespravna velikost */
      return;
    sizep = size;
    while (sizep > __MM_BRKINCR_TOPV)
    {
      brk(-__MM_BRKINCR_TOPV);         /* vlastni posun */
      sizep -= __MM_BRKINCR_TOPV;
    }
    if (sizep > 0)
      brk(-1*(int)sizep);
    __dynaend -= size;                 /* uprava konce d.p. dynaend */
  }
}

